﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Collections.Specialized;
using System.ComponentModel;
using System.Linq;
using System.Reflection;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Input;

namespace TestLibrary
{
    /// <summary>
    /// 셀렉터
    /// </summary>
    public class Selector : ItemsControl, IWeakEventListener
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Class
        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region 값 상등 비교자 - ValueEqualityComparer

        /// <summary>
        /// 값 상등 비교자
        /// </summary>
        private class ValueEqualityComparer : IEqualityComparer<string>
        {
            //////////////////////////////////////////////////////////////////////////////////////////////////// Method
            ////////////////////////////////////////////////////////////////////////////////////////// Public

            #region 상등 여부 구하기 - Equals(x, y)

            /// <summary>
            /// 상등 여부 구하기
            /// </summary>
            /// <param name="x">X 문자열</param>
            /// <param name="y">Y 문자열</param>
            /// <returns>상등 여부</returns>
            public bool Equals(string x, string y)
            {
                return string.Equals(x, y, StringComparison.InvariantCultureIgnoreCase);
            }

            #endregion
            #region 해시 코드 구하기 - GetHashCode(sourceObject)

            /// <summary>
            /// 해시 코드 구하기
            /// </summary>
            /// <param name="sourceObject">소스 객체</param>
            /// <returns>해시 코드</returns>
            public int GetHashCode(string sourceObject)
            {
                return 1;
            }

            #endregion
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Static
        //////////////////////////////////////////////////////////////////////////////// Public

        #region 선택시 이벤트 - SelectedEvent

        /// <summary>
        /// 선택시 이벤트
        /// </summary>
        public static readonly RoutedEvent SelectedEvent = EventManager.RegisterRoutedEvent("SelectedEvent", RoutingStrategy.Bubble, typeof(RoutedEventHandler), typeof(Selector));

        #endregion
        #region 선택 해제시 이벤트 - UnSelectedEvent

        /// <summary>
        /// 선택 해제시 이벤트
        /// </summary>
        public static readonly RoutedEvent UnSelectedEvent = EventManager.RegisterRoutedEvent("UnSelectedEvent", RoutingStrategy.Bubble, typeof(RoutedEventHandler), typeof(Selector));

        #endregion
        #region 항목 선택 변경시 이벤트 - ItemSelectionChangedEvent

        /// <summary>
        /// 항목 선택 변경시 이벤트
        /// </summary>
        public static readonly RoutedEvent ItemSelectionChangedEvent = EventManager.RegisterRoutedEvent("ItemSelectionChanged", RoutingStrategy.Bubble, typeof(ItemSelectionChangedEventHandler), typeof(Selector));

        #endregion

        #region 명령 의존 속성 - CommandProperty

        /// <summary>
        /// 명령 의존 속성
        /// </summary>
        public static readonly DependencyProperty CommandProperty = DependencyProperty.Register("Command", typeof(ICommand), typeof(Selector), new PropertyMetadata((ICommand)null));

        #endregion
        #region 구분자 의존 속성 - DelimiterProperty

        /// <summary>
        /// 구분자 의존 속성
        /// </summary>
        public static readonly DependencyProperty DelimiterProperty = DependencyProperty.Register("Delimiter", typeof(string), typeof(Selector), new UIPropertyMetadata(",", delimiterPropertyChangedCallback));

        #endregion
        #region 선택 항목 의존 속성 - SelectedItemProperty

        /// <summary>
        /// 선택 항목 의존 속성
        /// </summary>
        public static readonly DependencyProperty SelectedItemProperty = DependencyProperty.Register("SelectedItem", typeof(object), typeof(Selector), new UIPropertyMetadata(null, selectedItemPropertyChangedCallback));

        #endregion
        #region 선택 항목들 (오버라이드) 의존 속성 - SelectedItemsOverrideProperty

        /// <summary>
        /// 선택 항목들 (오버라이드) 의존 속성
        /// </summary>
        public static readonly DependencyProperty SelectedItemsOverrideProperty = DependencyProperty.Register("SelectedItemsOverride", typeof(IList), typeof(Selector), new UIPropertyMetadata(null, selectedItemsOverridePropertyChangedCallback));

        #endregion
        #region 선택 멤버 경로 의존 속성 - SelectedMemberPathProperty

        /// <summary>
        /// 선택 멤버 경로 의존 속성
        /// </summary>
        public static readonly DependencyProperty SelectedMemberPathProperty = DependencyProperty.Register("SelectedMemberPath", typeof(string), typeof(Selector), new UIPropertyMetadata(null, selectedMemberPathPropertyChangedCallback));

        #endregion
        #region 선택 값 의존 속성 - SelectedValueProperty

        /// <summary>
        /// 선택 값 의존 속성
        /// </summary>
        public static readonly DependencyProperty SelectedValueProperty = DependencyProperty.Register("SelectedValue", typeof(string), typeof(Selector), new FrameworkPropertyMetadata(null, FrameworkPropertyMetadataOptions.BindsTwoWayByDefault, selectedValuePropertyChangedCallback));

        #endregion
        #region 값 멤버 경로 의존 속성 - ValueMemberPathProperty

        /// <summary>
        /// 값 멤버 경로 의존 속성
        /// </summary>
        public static readonly DependencyProperty ValueMemberPathProperty = DependencyProperty.Register("ValueMemberPath", typeof(string), typeof(Selector), new UIPropertyMetadata(valueMemberPathPropertyChangedCallback));

        #endregion

        ////////////////////////////////////////////////////////////////////////////////////////// Instance
        //////////////////////////////////////////////////////////////////////////////// Protected

        #region Field

        /// <summary>
        /// 항목 선택 변경시 보류 여부
        /// </summary>
        protected bool surpressItemSelectionChanged;

        #endregion

        //////////////////////////////////////////////////////////////////////////////// Private

        #region Field

        /// <summary>
        /// 선택 항목 변경시 무시 여부
        /// </summary>
        private bool ignoreSelectedItemChanged;

        /// <summary>
        /// 선택 값 변경시 무시 여부
        /// </summary>
        private bool ignoreSelectedValueChanged;

        /// <summary>
        /// 선택 항목들 컬렉션 변경시 무시 카운트
        /// </summary>
        private int countIgnoreSelectedItemsCollectionChanged;

        /// <summary>
        /// 선택 멤버 경로 값 변경시 무시 카운트
        /// </summary>
        private int countIgnoreSelectedMemberPathValuesChanged;

        /// <summary>
        /// 선택 항목들
        /// </summary>
        private IList selectedItems;

        /// <summary>
        /// 제거 항목들
        /// </summary>
        private IList removedItems = new ObservableCollection<object>();

        /// <summary>
        /// 선택 멤버 경로 값 변경시 헬퍼
        /// </summary>
        private ValueChangedHelper selectedMemberPathValueChangedHelper;

        /// <summary>
        /// 값 멤버 경로 값 변경시 헬퍼
        /// </summary>
        private ValueChangedHelper valueMemberPathValueChangedHelper;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Event
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 항목 선택 변경시 - ItemSelectionChanged

        /// <summary>
        /// 항목 선택 변경시
        /// </summary>
        public event ItemSelectionChangedEventHandler ItemSelectionChanged
        {
            add
            {
                AddHandler(ItemSelectionChangedEvent, value);
            }
            remove
            {
                RemoveHandler(ItemSelectionChangedEvent, value);
            }
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Property
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 명령 - Command

        /// <summary>
        /// 명령
        /// </summary>
        [Category("Selector")]
        [Description("명령을 가져오거나 설정합니다.")]
        [TypeConverter(typeof(CommandConverter))]
        public ICommand Command
        {
            get
            {
                return (ICommand)GetValue(CommandProperty);
            }
            set
            {
                SetValue(CommandProperty, value);
            }
        }

        #endregion
        #region 구분자 - Delimiter

        /// <summary>
        /// 구분자
        /// </summary>
        public string Delimiter
        {
            get
            {
                return (string)GetValue(DelimiterProperty);
            }
            set
            {
                SetValue(DelimiterProperty, value);
            }
        }

        #endregion
        #region 선택 항목 - SelectedItem

        /// <summary>
        /// 선택 항목
        /// </summary>
        public object SelectedItem
        {
            get
            {
                return (object)GetValue(SelectedItemProperty);
            }
            set
            {
                SetValue(SelectedItemProperty, value);
            }
        }

        #endregion
        #region 선택 항목들 - SelectedItems

        /// <summary>
        /// 선택 항목들
        /// </summary>
        public IList SelectedItems
        {
            get
            {
                return this.selectedItems;
            }
            private set
            {
                if(value == null)
                {
                    throw new ArgumentNullException("value");
                }

                INotifyCollectionChanged oldNotifyCollectionChanged = this.selectedItems as INotifyCollectionChanged;
                INotifyCollectionChanged newNotifyCollectionChanged = value              as INotifyCollectionChanged;

                if(oldNotifyCollectionChanged != null)
                {
                    CollectionChangedEventManager.RemoveListener(oldNotifyCollectionChanged, this);
                }

                if(newNotifyCollectionChanged != null)
                {
                    CollectionChangedEventManager.AddListener(newNotifyCollectionChanged, this);
                }

                this.selectedItems = value;
            }
        }

        #endregion
        #region 선택 항목들 (오버라이드) - SelectedItemsOverride

        /// <summary>
        /// 선택 항목들 (오버라이드)
        /// </summary>
        public IList SelectedItemsOverride
        {
            get
            {
                return (IList)GetValue(SelectedItemsOverrideProperty);
            }
            set
            {
                SetValue(SelectedItemsOverrideProperty, value);
            }
        }

        #endregion
        #region 선택 멤버 경로 - SelectedMemberPath

        /// <summary>
        /// 선택 멤버 경로
        /// </summary>
        public string SelectedMemberPath
        {
            get
            {
                return (string)GetValue(SelectedMemberPathProperty);
            }
            set
            {
                SetValue(SelectedMemberPathProperty, value);
            }
        }

        #endregion
        #region 선택 값 - SelectedValue

        /// <summary>
        /// 선택 값
        /// </summary>
        public string SelectedValue
        {
            get
            {
                return (string)GetValue(SelectedValueProperty);
            }
            set
            {
                SetValue(SelectedValueProperty, value);
            }
        }

        #endregion
        #region 값 멤버 경로 - ValueMemberPath

        /// <summary>
        /// 값 멤버 경로
        /// </summary>
        public string ValueMemberPath
        {
            get
            {
                return (string)GetValue(ValueMemberPathProperty);
            }
            set
            {
                SetValue(ValueMemberPathProperty, value);
            }
        }

        #endregion

        ////////////////////////////////////////////////////////////////////////////////////////// Protected

        #region 항목들 컬렉션 - ItemsCollection

        /// <summary>
        /// 항목들 컬렉션
        /// </summary>
        protected IEnumerable ItemsCollection
        {
            get
            {
                return ItemsSource ?? ((IEnumerable)Items ?? (IEnumerable)new object[0]);
            }
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - Selector()

        /// <summary>
        /// 생성자
        /// </summary>
        public Selector()
        {
            SelectedItems = new ObservableCollection<object>();

            AddHandler(Selector.SelectedEvent  , new RoutedEventHandler((sender, e) => OnItemSelectionChangedCore(e, false)));
            AddHandler(Selector.UnSelectedEvent, new RoutedEventHandler((sender, e) => OnItemSelectionChangedCore(e, true )));

            this.selectedMemberPathValueChangedHelper = new ValueChangedHelper(OnSelectedMemberPathValuesChanged);
            this.valueMemberPathValueChangedHelper    = new ValueChangedHelper(OnValueMemberPathValuesChanged);
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Static
        //////////////////////////////////////////////////////////////////////////////// Private

        #region 구분자 속성 변경시 콜백 처리하기 - delimiterPropertyChangedCallback(dependencyObject, e)

        /// <summary>
        /// 구분자 속성 변경시 콜백 처리하기
        /// </summary>
        /// <param name="dependencyObject">의존 객체</param>
        /// <param name="e">이벤트 인자</param>
        private static void delimiterPropertyChangedCallback(DependencyObject dependencyObject, DependencyPropertyChangedEventArgs e)
        {
            ((Selector)dependencyObject).OnSelectedItemChanged((string)e.OldValue, (string)e.NewValue);
        }

        #endregion
        #region 선택 항목 속성 변경시 콜백 처리하기 - selectedItemPropertyChangedCallback(dependencyObject, e)

        /// <summary>
        /// 선택 항목 속성 변경시 콜백 처리하기
        /// </summary>
        /// <param name="dependencyObject">의존 객체</param>
        /// <param name="e">이벤트 인자</param>
        private static void selectedItemPropertyChangedCallback(DependencyObject dependencyObject, DependencyPropertyChangedEventArgs e)
        {
            ((Selector)dependencyObject).OnSelectedItemChanged(e.OldValue, e.NewValue);
        }

        #endregion
        #region 선택 항목들 (오버라이드) 속성 변경시 콜백 처리하기 - selectedItemsOverridePropertyChangedCallback(dependencyObject, e)

        /// <summary>
        /// 선택 항목들 (오버라이드) 속성 변경시 콜백 처리하기
        /// </summary>
        /// <param name="dependencyObject">의존 객체</param>
        /// <param name="e">이벤트 인자</param>
        private static void selectedItemsOverridePropertyChangedCallback(DependencyObject dependencyObject, DependencyPropertyChangedEventArgs e)
        {
            ((Selector)dependencyObject).OnSelectedItemsOverrideChanged((IList)e.OldValue, (IList)e.NewValue);
        }

        #endregion
        #region 선택 멤버 경로 속성 변경시 콜백 처리하기 - selectedMemberPathPropertyChangedCallback(dependencyObject, e)

        /// <summary>
        /// 선택 멤버 경로 속성 변경시 콜백 처리하기
        /// </summary>
        /// <param name="dependencyObject">의존 객체</param>
        /// <param name="e">이벤트 인자</param>
        private static void selectedMemberPathPropertyChangedCallback(DependencyObject dependencyObject, DependencyPropertyChangedEventArgs e)
        {
            ((Selector)dependencyObject).OnSelectedMemberPathChanged((string)e.OldValue, (string)e.NewValue);
        }

        #endregion
        #region 선택 값 속성 변경시 콜백 처리하기 - selectedValuePropertyChangedCallback(dependencyObject, e)

        /// <summary>
        /// 선택 값 속성 변경시 콜백 처리하기
        /// </summary>
        /// <param name="dependencyObject">의존 객체</param>
        /// <param name="e">이벤트 인자</param>
        private static void selectedValuePropertyChangedCallback(DependencyObject dependencyObject, DependencyPropertyChangedEventArgs e)
        {
            Selector selector = dependencyObject as Selector;

            if(selector != null)
            {
                selector.OnSelectedValueChanged((string)e.OldValue, (string)e.NewValue);
            }
        }

        #endregion
        #region 값 멤버 경로 속성 변경시 콜백 처리하기 - valueMemberPathPropertyChangedCallback(dependencyObject, e)

        /// <summary>
        /// 값 멤버 경로 속성 변경시 콜백 처리하기
        /// </summary>
        /// <param name="dependencyObject">의존 객체</param>
        /// <param name="e">이벤트 인자</param>
        private static void valueMemberPathPropertyChangedCallback(DependencyObject dependencyObject, DependencyPropertyChangedEventArgs e)
        {
            ((Selector)dependencyObject).OnValueMemberPathChanged((string)e.OldValue, (string)e.NewValue);
        }

        #endregion

        ////////////////////////////////////////////////////////////////////////////////////////// Instance
        //////////////////////////////////////////////////////////////////////////////// Internal

        #region 리스트에서 갱신하기 - UpdateFromList(selectedValues, getItemfunction)

        /// <summary>
        /// 리스트에서 갱신하기
        /// </summary>
        /// <param name="selectedValues">선택 값들</param>
        /// <param name="getItemfunction">항목 구하기 함수</param>
        internal void UpdateFromList(List<string> selectedValues, Func<object, object> getItemfunction)
        {
            this.countIgnoreSelectedItemsCollectionChanged++;

            SelectedItems.Clear();

            if((selectedValues != null) && (selectedValues.Count > 0))
            {
                ValueEqualityComparer comparer = new ValueEqualityComparer();

                foreach(object item in ItemsCollection)
                {
                    object itemValue = getItemfunction(item);

                    bool isSelected = (itemValue != null) && selectedValues.Contains(itemValue.ToString(), comparer);

                    if(isSelected)
                    {
                        SelectedItems.Add(item);
                    }
                }
            }

            this.countIgnoreSelectedItemsCollectionChanged--;

            UpdateFromSelectedItems();
        }

        #endregion
        #region 선택 항목들 갱신하기 - UpdateSelectedItems(selectedList)

        /// <summary>
        /// 선택 항목들 갱신하기
        /// </summary>
        /// <param name="selectedList">선택 리스트</param>
        internal void UpdateSelectedItems(IList selectedList)
        {
            if(selectedList == null)
            {
                throw new ArgumentNullException("selectedList");
            }

            if(selectedList.Count == this.SelectedItems.Count && selectedList.Cast<object>().SequenceEqual(this.SelectedItems.Cast<object>()))
            {
                return;
            }

            this.countIgnoreSelectedItemsCollectionChanged++;

            SelectedItems.Clear();

            foreach(object newItem in selectedList)
            {
                SelectedItems.Add(newItem);
            }

            this.countIgnoreSelectedItemsCollectionChanged--;

            UpdateFromSelectedItems();
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////// Public

        #region 초기화 종료하기 - EndInit()

        /// <summary>
        /// 초기화 종료하기
        /// </summary>
        public override void EndInit()
        {
            base.EndInit();

            if(SelectedItemsOverride != null)
            {
                OnSelectedItemsOverrideChanged(null, SelectedItemsOverride);
            }
            else if(SelectedMemberPath != null)
            {
                OnSelectedMemberPathChanged(null, SelectedMemberPath);
            }
            else if(SelectedValue != null)
            {
                OnSelectedValueChanged(null, SelectedValue);
            }
            else if(SelectedItem != null)
            {
                OnSelectedItemChanged(null, SelectedItem);
            }

            if(ValueMemberPath != null)
            {
                OnValueMemberPathChanged(null, ValueMemberPath);
            }
        }

        #endregion

        // IWeakEventListener
        #region 약한 이벤트 수신하기 - ReceiveWeakEvent(managerType, sender, e)

        /// <summary>
        /// 약한 이벤트 수신하기
        /// </summary>
        /// <param name="managerType">관리자 타입</param>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        /// <returns>처리 결과</returns>
        public bool ReceiveWeakEvent(Type managerType, object sender, EventArgs e)
        {
            if(managerType == typeof(CollectionChangedEventManager))
            {
                if(object.ReferenceEquals(this.selectedItems, sender))
                {
                    OnSelectedItemsCollectionChanged(sender, (NotifyCollectionChangedEventArgs)e);

                    return true;
                }
                else if(object.ReferenceEquals(ItemsCollection, sender))
                {
                    OnItemsSourceCollectionChanged(sender, (NotifyCollectionChangedEventArgs)e);

                    return true;
                }
            }

            return false;
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////// Protected

        #region 선택 항목 변경시 처리하기 - OnSelectedItemChanged(oldValue, newValue)

        /// <summary>
        /// 선택 항목 변경시 처리하기
        /// </summary>
        /// <param name="oldValue">기존 값</param>
        /// <param name="newValue">신규 값</param>
        protected virtual void OnSelectedItemChanged(string oldValue, string newValue)
        {
            if(!IsInitialized)
            {
                return;
            }

            UpdateSelectedValue();
        }

        #endregion
        #region 선택 항목 변경시 처리하기 - OnSelectedItemChanged(oldValue, newValue)

        /// <summary>
        /// 선택 항목 변경시 처리하기
        /// </summary>
        /// <param name="oldValue">기존 값</param>
        /// <param name="newValue">신규 값</param>
        protected virtual void OnSelectedItemChanged(object oldValue, object newValue)
        {
            if(!IsInitialized || this.ignoreSelectedItemChanged)
            {
                return;
            }

            this.countIgnoreSelectedItemsCollectionChanged++;

            SelectedItems.Clear();

            if(newValue != null)
            {
                SelectedItems.Add(newValue);
            }

            UpdateFromSelectedItems();

            this.countIgnoreSelectedItemsCollectionChanged--;
        }

        #endregion
        #region 선택 항목들 (오버라이드) 변경시 처리하기 - OnSelectedItemsOverrideChanged(oldValue, newValue)

        /// <summary>
        /// 선택 항목들 (오버라이드) 변경시 처리하기
        /// </summary>
        /// <param name="oldValue">기존 값</param>
        /// <param name="newValue">신규 값</param>
        protected virtual void OnSelectedItemsOverrideChanged(IList oldValue, IList newValue)
        {
            if(!IsInitialized)
            {
                return;
            }

            this.SelectedItems = (newValue != null) ? newValue : new ObservableCollection<object>();

            UpdateFromSelectedItems();
        }

        #endregion
        #region 선택 멤버 경로 변경시 처리하기 - OnSelectedMemberPathChanged(oldValue, newValue)
 
               /// <summary>
        /// 선택 멤버 경로 변경시 처리하기
        /// </summary>
        /// <param name="oldValue">기존 값</param>
        /// <param name="newValue">신규 값</param>
        protected virtual void OnSelectedMemberPathChanged(string oldValue, string newValue)
        {
            if(!IsInitialized)
            {
                return;
            }

            UpdateSelectedMemberPathValuesBindings();
        }

        #endregion
        #region 선택 값 변경시 처리하기 - OnSelectedValueChanged(oldValue, newValue)

        /// <summary>
        /// 선택 값 변경시 처리하기
        /// </summary>
        /// <param name="oldValue">기존 값</param>
        /// <param name="newValue">신규 값</param>
        protected virtual void OnSelectedValueChanged(string oldValue, string newValue)
        {
            if(!IsInitialized || this.ignoreSelectedValueChanged)
            {
                return;
            }

            UpdateFromSelectedValue();
        }

        #endregion
        #region 값 멤버 경로 변경시 처리하기 - OnValueMemberPathChanged(oldValue, newValue)

        /// <summary>
        /// 값 멤버 경로 변경시 처리하기
        /// </summary>
        /// <param name="oldValue">기존 값</param>
        /// <param name="newValue">신규 값</param>
        protected virtual void OnValueMemberPathChanged(string oldValue, string newValue)
        {
            if(!IsInitialized)
            {
                return;
            }

            UpdateValueMemberPathValuesBindings();
        }

        #endregion
        #region 항목들 소스 변경시 처리하기 - OnItemsSourceChanged(oldValue, newValue)

        /// <summary>
        /// 항목들 소스 변경시 처리하기
        /// </summary>
        /// <param name="oldValue">기존 값</param>
        /// <param name="newValue">신규 값</param>
        protected override void OnItemsSourceChanged(IEnumerable oldValue, IEnumerable newValue)
        {
            base.OnItemsSourceChanged(oldValue, newValue);

            INotifyCollectionChanged oldNotifyCollectionChanged = oldValue as INotifyCollectionChanged;
            INotifyCollectionChanged newNotifyCollectionChanged = newValue as INotifyCollectionChanged;

            if(oldNotifyCollectionChanged != null)
            {
                CollectionChangedEventManager.RemoveListener(oldNotifyCollectionChanged, this);
            }

            if(newNotifyCollectionChanged != null)
            {
                CollectionChangedEventManager.AddListener(newNotifyCollectionChanged, this);
            }

            if(!IsInitialized)
            {
                return;
            }

            if(!VirtualizingStackPanel.GetIsVirtualizing(this) || (VirtualizingStackPanel.GetIsVirtualizing(this) && (newValue != null)))
            {
                RemoveUnavailableSelectedItems();
            }

            UpdateSelectedMemberPathValuesBindings();
            UpdateValueMemberPathValuesBindings();
        }

        #endregion

        #region 선택 항목들 컬렉션 변경시 처리하기 - OnSelectedItemsCollectionChanged(sender, e)

        /// <summary>
        /// 선택 항목들 컬렉션 변경시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        protected virtual void OnSelectedItemsCollectionChanged(object sender, NotifyCollectionChangedEventArgs e)
        {
            if(this.countIgnoreSelectedItemsCollectionChanged > 0)
            {
                return;
            }

            UpdateFromSelectedItems();
        }

        #endregion

        #region 항목 선택 변경시 처리하기 - OnItemSelectionChanged(e)

        /// <summary>
        /// 항목 선택 변경시 처리하기
        /// </summary>
        /// <param name="e">이벤트 인자</param>
        protected virtual void OnItemSelectionChanged(ItemSelectionChangedEventArgs e)
        {
            if(this.surpressItemSelectionChanged)
            {
                return;
            }

            RaiseEvent(e);

            if(Command != null)
            {
                Command.Execute(e.Item);
            }
        }

        #endregion

        #region 항목 자신 컨테이너 여부 구하기 (오버라이드) - IsItemItsOwnContainerOverride(item)

        /// <summary>
        /// 항목 자신 컨테이너 여부 구하기 (오버라이드)
        /// </summary>
        /// <param name="item">항목</param>
        /// <returns>항목 자신 컨테이너 여부</returns>
        protected override bool IsItemItsOwnContainerOverride(object item)
        {
            return item is SelectorItem;
        }

        #endregion
        #region 항목용 컨테이너 구하기 (오버라이드) - GetContainerForItemOverride()

        /// <summary>
        /// 항목용 컨테이너 구하기 (오버라이드)
        /// </summary>
        /// <returns>항목용 컨테이너</returns>
        protected override DependencyObject GetContainerForItemOverride()
        {
            return new SelectorItem();
        }

        #endregion
        #region 항목용 컨테이너 준비하기 (오버라이드) - PrepareContainerForItemOverride(dependencyObject, item)

        /// <summary>
        /// 항목용 컨테이너 준비하기 (오버라이드)
        /// </summary>
        /// <param name="dependencyObject">의존 객체</param>
        /// <param name="item">항목</param>
        protected override void PrepareContainerForItemOverride(DependencyObject dependencyObject, object item)
        {
            base.PrepareContainerForItemOverride(dependencyObject, item);

            this.surpressItemSelectionChanged = true;

            FrameworkElement frameworkElement = dependencyObject as FrameworkElement;

            frameworkElement.SetValue(SelectorItem.IsSelectedProperty, SelectedItems.Contains(item));

            this.surpressItemSelectionChanged = false;
        }

        #endregion

        #region 속성 경로 값 구하기 - GetPropertyPathValue(item, propertyPath)

        /// <summary>
        /// 속성 경로 값 구하기
        /// </summary>
        /// <param name="item">항목</param>
        /// <param name="propertyPath">속성 경로</param>
        /// <returns>경로 값</returns>
        protected object GetPropertyPathValue(object item, string propertyPath)
        {
            if(item == null)
            {
                throw new ArgumentNullException("item");
            }

            if(string.IsNullOrEmpty(propertyPath) || propertyPath == ".")
            {
                return item;
            }

            PropertyInfo propertyInfo = item.GetType().GetProperty(propertyPath);

            return (propertyInfo != null) ? propertyInfo.GetValue(item, null) : null;
        }

        #endregion
        #region 항목 값 구하기 - GetItemValue(item)

        /// <summary>
        /// 항목 값 구하기
        /// </summary>
        /// <param name="item">항목</param>
        /// <returns>항목 값</returns>
        protected object GetItemValue(object item)
        {
            return (item != null) ? this.GetPropertyPathValue(item, this.ValueMemberPath) : null;
        }

        #endregion
        #region 값으로 항목 결정하기 - ResolveItemByValue(value)

        /// <summary>
        /// 값으로 항목 결정하기
        /// </summary>
        /// <param name="value">값</param>
        /// <returns>항목</returns>
        protected object ResolveItemByValue(string value)
        {
            if(!string.IsNullOrEmpty(ValueMemberPath))
            {
                foreach(object item in ItemsCollection)
                {
                    PropertyInfo propertyInfo = item.GetType().GetProperty(ValueMemberPath);

                    if(propertyInfo != null)
                    {
                        object propertyValue = propertyInfo.GetValue(item, null);

                        if(value.Equals(propertyValue.ToString(), StringComparison.InvariantCultureIgnoreCase))
                        {
                            return item;
                        }
                    }
                }
            }

            return value;
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////// Private

        #region 선택 멤버 경로 값 구하기 - GetSelectedMemberPathValue(item)

        /// <summary>
        /// 선택 멤버 경로 값 구하기
        /// </summary>
        /// <param name="item">항목</param>
        /// <returns>선택 멤버 경로 값</returns>
        private bool? GetSelectedMemberPathValue(object item)
        {
            PropertyInfo propertyInfo = this.GetSelectedMemberPathProperty(item);

            return (propertyInfo != null) ? (bool)propertyInfo.GetValue(item, null) : (bool?)null;
        }

        #endregion
        #region 선택 멤버 경로 값 설정하기 - SetSelectedMemberPathValue(item, value)

        /// <summary>
        /// 선택 멤버 경로 값 설정하기
        /// </summary>
        /// <param name="item">항목</param>
        /// <param name="value">값</param>
        private void SetSelectedMemberPathValue(object item, bool value)
        {
            PropertyInfo propertyInfo = GetSelectedMemberPathProperty(item);

            if(propertyInfo != null)
            {
                propertyInfo.SetValue(item, value, null);
            }
        }

        #endregion
        #region 선택 멤버 경로 속성 구하기 - GetSelectedMemberPathProperty(item)

        /// <summary>
        /// 선택 멤버 경로 속성 구하기
        /// </summary>
        /// <param name="item">항목</param>
        /// <returns>선택 멤버 경로 속성</returns>
        private PropertyInfo GetSelectedMemberPathProperty(object item)
        {
            if(!string.IsNullOrEmpty(SelectedMemberPath) && (item != null))
            {
                PropertyInfo propertyInfo = item.GetType().GetProperty(SelectedMemberPath);

                if(propertyInfo != null && propertyInfo.PropertyType == typeof(bool))
                {
                    return propertyInfo;
                }
            }

            return null;
        }

        #endregion

        #region 항목 선택 변경시 처리하기 (코어) - OnItemSelectionChangedCore(e, unselected)

        /// <summary>
        /// 항목 선택 변경시 처리하기 (코어)
        /// </summary>
        /// <param name="e">이벤트 인자</param>
        /// <param name="unselected">선택 해제 여부</param>
        private void OnItemSelectionChangedCore(RoutedEventArgs e, bool unselected)
        {
            object item = this.ItemContainerGenerator.ItemFromContainer((DependencyObject)e.OriginalSource);

            if(item == DependencyProperty.UnsetValue)
            {
                item = e.OriginalSource;
            }

            if(unselected)
            {
                while(SelectedItems.Contains(item))
                {
                    SelectedItems.Remove(item);
                }
            }
            else
            {
                if(!SelectedItems.Contains(item))
                {
                    SelectedItems.Add(item);
                }
            }

            OnItemSelectionChanged(new ItemSelectionChangedEventArgs(Selector.ItemSelectionChangedEvent, this, item, !unselected));
        }

        #endregion
        #region 항목들 소스 컬렉션 변경시 처리하기 - OnItemsSourceCollectionChanged(sender, e)

        /// <summary>
        /// 항목들 소스 컬렉션 변경시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void OnItemsSourceCollectionChanged(object sender, NotifyCollectionChangedEventArgs e)
        {
            RemoveUnavailableSelectedItems();

            AddAvailableRemovedItems();

            UpdateSelectedMemberPathValuesBindings();

            UpdateValueMemberPathValuesBindings();
        }

        #endregion
        #region 선택 멤버 경로 값들 변경시 처리하기 - OnSelectedMemberPathValuesChanged()

        /// <summary>
        /// 선택 멤버 경로 값들 변경시 처리하기
        /// </summary>
        private void OnSelectedMemberPathValuesChanged()
        {
            if(this.countIgnoreSelectedMemberPathValuesChanged > 0)
            {
                return;
            }

            UpdateFromSelectedMemberPathValues();
        }

        #endregion
        #region 값 멤버 경로 값들 변경시 처리하기 - OnValueMemberPathValuesChanged()

        /// <summary>
        /// 값 멤버 경로 값들 변경시 처리하기
        /// </summary>
        private void OnValueMemberPathValuesChanged()
        {
            UpdateSelectedValue();
        }

        #endregion

        #region 선택 멤버 경로 값들의 바인딩들 갱신하기 - UpdateSelectedMemberPathValuesBindings()

        /// <summary>
        /// 선택 멤버 경로 값들의 바인딩들 갱신하기
        /// </summary>
        private void UpdateSelectedMemberPathValuesBindings()
        {
            this.selectedMemberPathValueChangedHelper.UpdateValueSource(ItemsCollection, SelectedMemberPath);
        }

        #endregion
        #region 값 멤버 경로 값들의 바인딩들 갱신하기 - UpdateValueMemberPathValuesBindings()

        /// <summary>
        /// 값 멤버 경로 값들의 바인딩들 갱신하기
        /// </summary>
        private void UpdateValueMemberPathValuesBindings()
        {
            this.valueMemberPathValueChangedHelper.UpdateValueSource(ItemsCollection, ValueMemberPath);
        }

        #endregion

        #region 선택 값 갱신하기 - UpdateSelectedValue()

        /// <summary>
        /// 선택 값 갱신하기
        /// </summary>
        private void UpdateSelectedValue()
        {
            string newValue = string.Join(Delimiter, SelectedItems.Cast<object>().Select(x => GetItemValue(x)));

            if(string.IsNullOrEmpty(SelectedValue) || !SelectedValue.Equals(newValue))
            {
                this.ignoreSelectedValueChanged = true;

                SelectedValue = newValue;

                this.ignoreSelectedValueChanged = false;
            }
        }

        #endregion
        #region 선택 항목 갱신하기 - UpdateSelectedItem()

        /// <summary>
        /// 선택 항목 갱신하기
        /// </summary>
        private void UpdateSelectedItem()
        {
            if(!SelectedItems.Contains(SelectedItem))
            {
                this.ignoreSelectedItemChanged = true;

                SelectedItem = (SelectedItems.Count > 0) ? SelectedItems[0] : null;

                this.ignoreSelectedItemChanged = false;
            }
        }

        #endregion
        #region 선택 멤버 경로 값들에서 갱신하기 - UpdateFromSelectedMemberPathValues()

        /// <summary>
        /// 선택 멤버 경로 값들에서 갱신하기
        /// </summary>
        private void UpdateFromSelectedMemberPathValues()
        {
            this.countIgnoreSelectedItemsCollectionChanged++;

            foreach(object item in ItemsCollection)
            {
                bool? isSelected = GetSelectedMemberPathValue(item);

                if(isSelected != null)
                {
                    if(isSelected.Value)
                    {
                        if(!SelectedItems.Contains(item))
                        {
                            SelectedItems.Add(item);
                        }
                    }
                    else
                    {
                        if(SelectedItems.Contains(item))
                        {
                            SelectedItems.Remove(item);
                        }
                    }
                }
            }

            this.countIgnoreSelectedItemsCollectionChanged--;

            UpdateFromSelectedItems();
        }

        #endregion
        #region 선택 항목들에서 갱신하기 - UpdateFromSelectedItems()

        /// <summary>
        /// 선택 항목들에서 갱신하기
        /// </summary>
        private void UpdateFromSelectedItems()
        {
            foreach(object item in ItemsCollection)
            {
                bool isSelected = SelectedItems.Contains(item);

                this.countIgnoreSelectedMemberPathValuesChanged++;

                SetSelectedMemberPathValue(item, isSelected);

                this.countIgnoreSelectedMemberPathValuesChanged--;

                SelectorItem selectorItem = ItemContainerGenerator.ContainerFromItem(item) as SelectorItem;

                if(selectorItem != null)
                {
                    selectorItem.IsSelected = isSelected;
                }
            }

            UpdateSelectedItem();

            UpdateSelectedValue();
        }

        #endregion
        #region 선택 값에서 갱신하기 - UpdateFromSelectedValue()

        /// <summary>
        /// 선택 값에서 갱신하기
        /// </summary>
        private void UpdateFromSelectedValue()
        {
            List<string> selectedValueList = null;

            if(!string.IsNullOrEmpty(SelectedValue))
            {
                selectedValueList = SelectedValue.Split(new string[] { Delimiter }, StringSplitOptions.RemoveEmptyEntries).ToList();
            }

            UpdateFromList(selectedValueList, GetItemValue);
        }

        #endregion

        #region 이용 불가한 선택 항목들 제거하기 - RemoveUnavailableSelectedItems()

        /// <summary>
        /// 이용 불가한 선택 항목들 제거하기
        /// </summary>
        private void RemoveUnavailableSelectedItems()
        {
            this.countIgnoreSelectedItemsCollectionChanged++;

            HashSet<object> hashSet = new HashSet<object>(ItemsCollection.Cast<object>());

            for(int i = 0; i < SelectedItems.Count; i++)
            {
                if(!hashSet.Contains(SelectedItems[ i ]))
                {
                    this.removedItems.Add(SelectedItems[ i ]);

                    SelectedItems.RemoveAt(i);

                    i--;
                }
            }

            this.countIgnoreSelectedItemsCollectionChanged--;

            UpdateSelectedItem();

            UpdateSelectedValue();
        }

        #endregion
        #region 이용 가능한 제거 항목들 추가하기 - AddAvailableRemovedItems()

        /// <summary>
        /// 이용 가능한 제거 항목들 추가하기
        /// </summary>
        private void AddAvailableRemovedItems()
        {
            HashSet<object> hashSet = new HashSet<object>(ItemsCollection.Cast<object>());

            for(int i = 0; i < this.removedItems.Count; i++)
            {
                if(hashSet.Contains(this.removedItems[ i ]))
                {
                    SelectedItems.Add(this.removedItems[ i ]);

                    this.removedItems.RemoveAt(i);          

                    i--;
                }
            }
        }

        #endregion
    }

    /// <summary>
    /// 항목 선택 변경시 이벤트 핸들러
    /// </summary>
    /// <param name="sender">이벤트 발생자</param>
    /// <param name="e">이벤트 인자</param>
    public delegate void ItemSelectionChangedEventHandler(object sender, ItemSelectionChangedEventArgs e);

    /// <summary>
    /// 항목 선택 변경시 이벤트 인자
    /// </summary>
    public class ItemSelectionChangedEventArgs : RoutedEventArgs
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Property
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 선택 여부 - IsSelected

        /// <summary>
        /// 선택 여부
        /// </summary>
        public bool IsSelected { get; private set; }

        #endregion
        #region 항목 - Item

        /// <summary>
        /// 항목
        /// </summary>
        public object Item { get; private set; }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - ItemSelectionChangedEventArgs(routedEvent, source, item, isSelected)

        /// <summary>
        /// 생성자
        /// </summary>
        /// <param name="routedEvent">라우팅 이벤트</param>
        /// <param name="source">소스</param>
        /// <param name="item">항목</param>
        /// <param name="isSelected">선택 여부</param>
        public ItemSelectionChangedEventArgs(RoutedEvent routedEvent, object source, object item, bool isSelected) : base(routedEvent, source)
        {
            Item       = item;
            IsSelected = isSelected;
        }

        #endregion
    }
}