﻿using System.Windows;
using System.Windows.Controls;

namespace TestLibrary
{
    /// <summary>
    /// 선택자 항목
    /// </summary>
    public class SelectorItem : ContentControl
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Static
        //////////////////////////////////////////////////////////////////////////////// Public

        #region 선택시 이벤트 - SelectedEvent

        /// <summary>
        /// 선택시 이벤트
        /// </summary>
        public static readonly RoutedEvent SelectedEvent = Selector.SelectedEvent.AddOwner(typeof(SelectorItem));

        #endregion
        #region 선택 해제시 이벤트 - UnselectedEvent

        /// <summary>
        /// 선택 해제시 이벤트
        /// </summary>
        public static readonly RoutedEvent UnselectedEvent = Selector.UnSelectedEvent.AddOwner(typeof(SelectorItem));

        #endregion

        #region 선택 여부 의존 속성 - IsSelectedProperty

        /// <summary>
        /// 선택 여부 의존 속성
        /// </summary>
        public static readonly DependencyProperty IsSelectedProperty = DependencyProperty.Register("IsSelected", typeof(bool), typeof(SelectorItem), new UIPropertyMetadata(false, isSelectedPropertyChangedCallback));

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Property
        ////////////////////////////////////////////////////////////////////////////////////////// Internal

        #region 셀렉터 - Selector

        /// <summary>
        /// 셀렉터
        /// </summary>
        internal Selector Selector
        {
            get
            {
                return ItemsControl.ItemsControlFromItemContainer(this) as Selector;
            }
        }

        #endregion

        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 선택 여부 - IsSelected

        /// <summary>
        /// 선택 여부
        /// </summary>
        public bool IsSelected
        {
            get
            {
                return (bool)GetValue(IsSelectedProperty);
            }
            set
            {
                SetValue(IsSelectedProperty, value);
            }
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Static

        #region 생성자 - SelectorItem()

        /// <summary>
        /// 생성자
        /// </summary>
        static SelectorItem()
        {
            DefaultStyleKeyProperty.OverrideMetadata(typeof(SelectorItem), new FrameworkPropertyMetadata(typeof(SelectorItem)));
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Static
        //////////////////////////////////////////////////////////////////////////////// Private

        #region 선택 여부 속성 변경시 콜백 처리하기 - isSelectedPropertyChangedCallback(dependencyObject, e)

        /// <summary>
        /// 선택 여부 속성 변경시 콜백 처리하기
        /// </summary>
        /// <param name="dependencyObject">의존 객체</param>
        /// <param name="e">이벤트 인자</param>
        private static void isSelectedPropertyChangedCallback(DependencyObject dependencyObject, DependencyPropertyChangedEventArgs e)
        {
            SelectorItem selectorItem = dependencyObject as SelectorItem;

            if (selectorItem != null)
            {
                selectorItem.OnIsSelectedChanged((bool)e.OldValue, (bool)e.NewValue);
            }
        }

        #endregion

        ////////////////////////////////////////////////////////////////////////////////////////// Instance
        //////////////////////////////////////////////////////////////////////////////// Protected

        #region 선택 여부 변경시 처리하기 - OnIsSelectedChanged(oldValue, newValue)

        /// <summary>
        /// 선택 여부 변경시 처리하기
        /// </summary>
        /// <param name="oldValue">기존 값</param>
        /// <param name="newValue">신규 값</param>
        protected virtual void OnIsSelectedChanged(bool oldValue, bool newValue)
        {
            if(newValue)
            {
                RaiseEvent(new RoutedEventArgs(Selector.SelectedEvent, this));
            }
            else
            {
                RaiseEvent(new RoutedEventArgs(Selector.UnSelectedEvent, this));
            }
        }

        #endregion
    }
}