﻿using System;
using System.Collections;
using System.Globalization;
using System.Windows;
using System.Windows.Data;

namespace TestLibrary
{
    /// <summary>
    /// 값 변경시 헬퍼
    /// </summary>
    internal class ValueChangedHelper : DependencyObject
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Class
        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region 블랭크 멀티 값 변환자 - BlankMultiValueConverter

        /// <summary>
        /// 블랭크 멀티 값 변환자
        /// </summary>
        private class BlankMultiValueConverter : IMultiValueConverter
        {
            //////////////////////////////////////////////////////////////////////////////////////////////////// Method
            ////////////////////////////////////////////////////////////////////////////////////////// Public

            #region 변환하기 - Convert(sourceValueArray, targetType, parameter, cultureInfo)

            /// <summary>
            /// 변환하기
            /// </summary>
            /// <param name="sourceValueArray">소스 값 배열</param>
            /// <param name="targetType">타겟 타입</param>
            /// <param name="parameter">매개 변수</param>
            /// <param name="cultureInfo">문화 정보</param>
            /// <returns>변환 값</returns>
            public object Convert(object[] sourceValueArray, Type targetType, object parameter, CultureInfo cultureInfo)
            {
                return new object();
            }

            #endregion
            #region 역변환하기 - ConvertBack(sourceValue, targetTypeArray, parameter, cultureInfo)

            /// <summary>
            /// 역변환하기
            /// </summary>
            /// <param name="sourceValue">소스 값</param>
            /// <param name="targetTypeArray">타겟 타입 배열</param>
            /// <param name="parameter">매개 변수</param>
            /// <param name="cultureInfo">문화 정보</param>
            /// <returns>역변환 값</returns>
            public object[] ConvertBack(object sourceValue, Type[] targetTypeArray, object parameter, CultureInfo cultureInfo)
            {
                throw new InvalidOperationException();
            }

            #endregion
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Event
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 값 변경시 - ValueChanged

        /// <summary>
        /// 값 변경시
        /// </summary>
        public event EventHandler ValueChanged;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Static
        //////////////////////////////////////////////////////////////////////////////// Private

        #region 값 의존 속성 - ValueProperty

        /// <summary>
        /// 값 의존 속성
        /// </summary>
        private static readonly DependencyProperty ValueProperty = DependencyProperty.Register("Value", typeof(object), typeof(ValueChangedHelper), new UIPropertyMetadata(null, valuePropertyChangedCallback));

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Property
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 값 - Value

        /// <summary>
        /// 값
        /// </summary>
        private object Value
        {
            get
            {
                return (object)GetValue(ValueProperty);
            }
            set
            {
                SetValue(ValueProperty, value);
            }
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - ValueChangeHelper(valueChangedAction)

        /// <summary>
        /// 생성자
        /// </summary>
        /// <param name="valueChangedAction">값 변경시 액션</param>
        public ValueChangedHelper(Action valueChangedAction)
        {
            if(valueChangedAction == null)
            {
                throw new ArgumentNullException("changeCallbackAction");
            }

            ValueChanged += (sender, e) => valueChangedAction();
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Static
        //////////////////////////////////////////////////////////////////////////////// Private

        #region 값 속성 변경시 콜백 처리하기 - valuePropertyChangedCallback(dependencyObject, e)

        /// <summary>
        /// 값 속성 변경시 콜백 처리하기
        /// </summary>
        /// <param name="dependencyObject">의존 객체</param>
        /// <param name="e">이벤트 인자</param>
        private static void valuePropertyChangedCallback(DependencyObject dependencyObject, DependencyPropertyChangedEventArgs e)
        {
            ((ValueChangedHelper)dependencyObject).FireValueChangedEvent();
        }

        #endregion

        ////////////////////////////////////////////////////////////////////////////////////////// Instance
        //////////////////////////////////////////////////////////////////////////////// Public

        #region 값 소스 갱신하기 - UpdateValueSource(sourceItem, bindingPath)

        /// <summary>
        /// 값 소스 갱신하기
        /// </summary>
        /// <param name="sourceItem">소스 항목</param>
        /// <param name="bindingPath">바인딩 경로</param>
        public void UpdateValueSource(object sourceItem, string bindingPath)
        {
            BindingBase bindingBase = null;

            if(sourceItem != null && bindingPath != null)
            {
                bindingBase = new Binding(bindingPath) { Source = sourceItem };
            }

            UpdateBinding(bindingBase);
        }

        #endregion
        #region 값 소스 갱신하기 - UpdateValueSource(sourceItems, bindingPath)

        /// <summary>
        /// 값 소스 갱신하기
        /// </summary>
        /// <param name="sourceItems">소스 항목들</param>
        /// <param name="bindingPath">바인딩 경로</param>
        public void UpdateValueSource(IEnumerable sourceItems, string bindingPath)
        {
            BindingBase bindingBase = null;

            if(sourceItems != null && bindingPath != null)
            {
                MultiBinding multiBinding = new MultiBinding();

                multiBinding.Converter = new BlankMultiValueConverter();

                foreach(object item in sourceItems)
                {
                    multiBinding.Bindings.Add(new Binding(bindingPath) { Source = item });
                }

                bindingBase = multiBinding;
            }

            UpdateBinding(bindingBase);
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////// Private

        #region 바인딩 지우기 - ClearBinding()

        /// <summary>
        /// 바인딩 지우기
        /// </summary>
        private void ClearBinding()
        {
            BindingOperations.ClearBinding(this, ValueChangedHelper.ValueProperty);
        }

        #endregion
        #region 바인딩 갱신하기 - UpdateBinding(bindingBase)

        /// <summary>
        /// 바인딩 갱신하기
        /// </summary>
        /// <param name="bindingBase">바인딩 베이스</param>
        private void UpdateBinding(BindingBase bindingBase)
        {
            if(bindingBase != null)
            {
                BindingOperations.SetBinding(this, ValueChangedHelper.ValueProperty, bindingBase);
            }
            else
            {
                ClearBinding();
            }
        }

        #endregion

        #region 값 변경시 이벤트 발생시키기 - FireValueChangedEvent()

        /// <summary>
        /// 값 변경시 이벤트 발생시키기
        /// </summary>
        private void FireValueChangedEvent()
        {
            if(ValueChanged != null)
            {
                ValueChanged(this, EventArgs.Empty);
            }
        }

        #endregion
    }
}